"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.after = exports.before = void 0;
const ts = require("typescript");
const getSchemaByType_1 = require("../util/getSchemaByType");
const convert_1 = require("../util/convert");
const util_1 = require("../util");
const METADATA = {};
console.log('[egg-controller] load transformer: response-schema.');
function transformer(program) {
    const typeChecker = program.getTypeChecker();
    return {
        before(ctx) {
            return (sourceFile) => {
                before(ctx, sourceFile, typeChecker);
                return util_1.walker(sourceFile, ctx);
            };
        },
        after(ctx) {
            return (sourceFile) => {
                after(ctx, sourceFile);
                return util_1.walker(sourceFile, ctx);
            };
        },
    };
}
exports.default = transformer;
function before(_, sourceFile, typeChecker) {
    // 当前编译文件需要保存的数据
    const fileData = (METADATA[sourceFile.fileName] = {
        // 方法定义
        methodDefine: {},
        schemaObjects: {},
        typeCache: [],
    });
    // 遍历文件中所有的 class
    sourceFile.statements
        .filter(n => ts.isClassDeclaration(n))
        .forEach(node => {
        node.forEachChild(cbNode => {
            if (util_1.isDecoratorNameInclude(cbNode, 'route')) {
                const decorator = cbNode.decorators.find(dec => {
                    return dec.expression.expression.escapedText === 'route';
                });
                const clsName = node.name.text;
                const clsMethod = util_1.getClsMethodKey(cbNode.parent.symbol.escapedName, cbNode.symbol.escapedName);
                //#region find schema prop
                let expression = decorator.expression;
                if (!expression.arguments.length ||
                    !ts.isObjectLiteralExpression(expression.arguments[expression.arguments.length - 1])) {
                    expression.arguments = ts.createNodeArray([
                        ...expression.arguments,
                        ts.createObjectLiteral([], false),
                    ]);
                }
                const routeArg = expression.arguments[expression.arguments.length - 1];
                const schemaProp = getField(routeArg, 'schema');
                //#endregion
                const config = {
                    typeChecker,
                    schemaObjects: fileData.schemaObjects,
                    typeCache: fileData.typeCache,
                };
                const type = typeChecker.getTypeAtLocation(cbNode);
                const callSignatures = type.getCallSignatures()[0];
                // params parser
                if (!hasField(schemaProp, 'requestBody')) {
                    try {
                        const parameters = callSignatures.getParameters();
                        const paramSchema = parameters.reduce((s, p) => {
                            const paramType = typeChecker.getTypeAtLocation(p.valueDeclaration);
                            s[`${p.escapedName}`] = getSchemaByType_1.getSchemaByType(paramType, config);
                            return s;
                        }, {});
                        schemaProp.properties = ts.createNodeArray([
                            ...schemaProp.properties,
                            ts.createPropertyAssignment('requestBody', convert_1.convert({ type: 'object', properties: paramSchema })),
                        ]);
                    }
                    catch (error) {
                        console.log('warn:', `parse ${clsName}.${clsMethod} params fail!`, error);
                    }
                }
                // returnType parser
                if (!hasField(schemaProp, 'response')) {
                    try {
                        const returnType = typeChecker.getReturnTypeOfSignature(callSignatures);
                        let responseSchema;
                        if (util_1.getValue(() => returnType.symbol.escapedName) === 'Promise') {
                            responseSchema = getSchemaByType_1.getSchemaByType(returnType.typeArguments[0], {
                                typeChecker,
                                schemaObjects: fileData.schemaObjects,
                                typeCache: fileData.typeCache,
                            });
                        }
                        else {
                            responseSchema = getSchemaByType_1.getSchemaByType(returnType, config);
                        }
                        schemaProp.properties = ts.createNodeArray([
                            ...schemaProp.properties,
                            ts.createPropertyAssignment('response', convert_1.convert(responseSchema)),
                        ]);
                    }
                    catch (error) {
                        console.log('warn:', `parse ${clsName}.${clsMethod} returnType fail!`, error);
                    }
                }
                if (!hasField(schemaProp, 'components')) {
                    schemaProp.properties = ts.createNodeArray([
                        ...schemaProp.properties,
                        ts.createPropertyAssignment('components', ts.createIdentifier('__SchemaDefinition')),
                    ]);
                }
            }
        });
    });
}
exports.before = before;
function after(_, sourceFile) {
    const fileData = METADATA[sourceFile.fileName];
    if (fileData) {
        sourceFile.statements = ts.createNodeArray([
            ts.createVariableStatement([], ts.createVariableDeclarationList([
                ts.createVariableDeclaration('__SchemaDefinition', undefined, convert_1.convert(fileData.schemaObjects)),
            ], ts.NodeFlags.Const)),
            ...sourceFile.statements,
        ]);
    }
}
exports.after = after;
function hasField(config, fieldName) {
    return !!config.properties.find(p => {
        return ts.isIdentifier(p.name) && p.name.escapedText === fieldName;
    });
}
function getField(config, fieldName) {
    let field;
    field = config.properties.find(p => {
        return ts.isIdentifier(p.name) && p.name.escapedText === fieldName;
    });
    if (!field) {
        field = ts.createObjectLiteral();
        config.properties = ts.createNodeArray([
            ...config.properties,
            ts.createPropertyAssignment(fieldName, field),
        ]);
    }
    else {
        field = field.initializer;
    }
    return field;
}
